# coding:utf-8
"""
Иструменты для обновления КЛАДРа
@author: Fattakhov M. Z.
"""
import logging
import os
import traceback
import urllib2
import tempfile
import shutil

from django.conf import settings
from django.utils.translation import ugettext as _
from py7zlib import Archive7z
from suds.client import Client

logging.basicConfig(level=logging.ERROR)


class KLADRUpdaterException(Exception):
    """Ошибка обновления."""


class KLADRUpdater(object):
    """KLADR Updater object, obviously

    :keyword load_type: тип загрузки КЛАДРа.
    :keyword loaded_file: файл-архив КЛАДРа.
    """
    #: Типы загрузки КЛАДРа
    FILE_UPDATE = 1
    SERVICE_UPDATE = 2

    #: Результаты обновления КЛАДРа
    SUCCESS = 0
    FAIL = 1

    def __init__(self, load_type, loaded_file=None):
        super(KLADRUpdater, self).__init__()
        self.load_type = int(load_type)
        self.logger = logging.getLogger('kladr.updater')

        if load_type == self.SERVICE_UPDATE:
            self.address = getattr(settings, 'KLADR_UPDATE_URL')
            self.file_name = getattr(settings, 'KLADR_FILE_NAME')
            self.proxy = getattr(settings, 'PROXY', None)
            self.get_update_url()
            self.load_from_ws()

        elif load_type == self.FILE_UPDATE:
            self.load_from_local(loaded_file)

    def get_update_url(self):
        """Подключение к сервису и получение адреса
        последнего обновленного КЛАДРа.
        """
        try:
            client = Client(self.address, proxy=self.proxy)
            self.update_url = client.service.GetLastDownloadFileInfo()[self.file_name]
        except Exception, error:
            self.logger.error(error)
            raise KLADRUpdaterException(_(u"Невозможно подключиться к сервису обновлений. Проверьте настройки."))

    def load_from_ws(self):
        """Скачивание файла КЛАДР."""
        if self.proxy:
            proxy_handler = urllib2.ProxyHandler(self.proxy)
            proxy_opener = urllib2.build_opener(proxy_handler)
            urllib2.install_opener(proxy_opener)
        try:
            connection = urllib2.urlopen(self.update_url)
        except (urllib2.HTTPError, urllib2.URLError), e:
            self.logger.error(e.code)
            raise KLADRUpdaterException(_(u"Невозможно скачать файл обновлений. Ошибка urllib2."))
        except Exception:
            self.logger.error(traceback.format_exc())
            raise KLADRUpdaterException(_(u"Невозможно скачать файл обновлений."))
        else:
            self.file_dir = tempfile.mkdtemp()
            self.file_path = os.path.abspath(os.path.join(self.file_dir, self.update_url.split('/')[-1]))
            with open(self.file_path, 'wb') as archived_file:
                archived_file.write(connection.read())

    def load_from_local(self, temp_file):
        """Загрузка локального файла обновлений"""
        try:
            self.file_dir = tempfile.mkdtemp()
            self.file_path = os.path.abspath(os.path.join(self.file_dir, "kladr_updater"))
            with open(self.file_path, 'wb') as archived_file:
                archived_file.write(temp_file)
        except IOError, err:
            self.logger.error(err)
            raise KLADRUpdaterException(_(u"Ошибка при загруке файла файла."))

    def unzip_dbf_files(self):
        """Распаковка архива."""
        try:
            with open(self.file_path, 'rb') as archive_file:
                archive = Archive7z(archive_file)
                for out_name in archive.getnames():
                    out_file_path = os.path.join(self.file_dir, out_name)
                    with open(out_file_path, 'wb') as outfile:
                        outfile.write(archive.getmember(out_name).read())
        except Exception, err:
            self.logger.error(err)
            raise KLADRUpdaterException(_(u"Ошибка при разархивации файлов обновлений."))

    def clean(self):
        """Удаление всех временных файлов и т.д."""
        shutil.rmtree(self.file_dir)